import Logger from "@home-gallery/logger";
import { promisify, callbackify } from "@home-gallery/common";
import { readIndex } from "./read.js";
import { createIndex } from "./create.js";
import { updateIndex } from "./update.js";
import { writeIndex } from "./write.js";
import { checksum } from "./checksum.js";
import { createFilter } from "./filter/index.js";
import { createJournal, readJournal as readJournalAsync } from "./journal.js";
import { access } from "fs/promises";
const asyncChecksum = promisify(checksum);
const log = Logger("index");
function isLimitExeeded(filter) {
  return typeof filter.limitExceeded == "function" ? filter.limitExceeded() : false;
}
async function asyncCreateOrUpdate(directory, filename, options) {
  const now = new Date();
  const fileIndex = await readIndex(filename);
  const filter = await createFilter(fileIndex.data, options);
  const fsEntries = await createIndex(directory, { ...options, filter });
  const [updatedEntries, changes] = await updateIndex(fileIndex.data, fsEntries, options.matcherFn);
  const limitExceeded = isLimitExeeded(filter);
  log.info(now, `Collected file index changes`);
  return [fileIndex, updatedEntries, changes, limitExceeded];
}
function onlyChangesWithSha1sum(changes) {
  return {
    adds: changes.adds.filter((e) => e.sha1sum),
    changes: changes.changes.filter((e) => e.sha1sum),
    removes: changes.removes.filter((e) => e.sha1sum)
  };
}
function getFileEntriesForChecksum(updatedEntries, changes, options) {
  const fileEntries = updatedEntries.filter((e) => e.isFile);
  if (!options.journal || !changes) {
    return fileEntries;
  }
  const journalFileEntries = [
    ...changes.adds.filter((e) => e.isFile),
    ...changes.changes.filter((e) => e.isFile)
  ];
  const journalFilenames = journalFileEntries.map((e) => e.filename);
  const addChecksumEntries = fileEntries.filter((e) => !e.sha1sum && !journalFilenames.includes(e.filename));
  journalFileEntries.push(...addChecksumEntries);
  return journalFileEntries;
}
async function calculateChecksum(directory, updatedEntries, changes, options) {
  if (!options.checksum) {
    return [[], false];
  }
  const fileEntries = getFileEntriesForChecksum(updatedEntries, changes, options);
  const checksumEntries = fileEntries.filter((e) => !e.sha1sum);
  const sha1sumDate = new Date().toISOString();
  return asyncChecksum(directory, fileEntries, checksumEntries, sha1sumDate);
}
function isUnchanged(changes) {
  return !changes.adds.length && !changes.changes.length && !changes.removes.length;
}
async function update(directory, filename, options) {
  const t0 = Date.now();
  log.info(`Updating file index for directory ${directory}`);
  const [index, updatedEntries, changes, limitExceeded] = await asyncCreateOrUpdate(directory, filename, options);
  const [updatedChecksumEntries, interrupted] = await calculateChecksum(directory, updatedEntries, changes, options);
  if (index.data.length && isUnchanged(changes) && !updatedChecksumEntries.length) {
    if (!options.journal) {
      log.info(t0, `No file changes in directory ${directory}`);
      return [index, false, limitExceeded];
    }
    const journal2 = await createJournal(filename, index, updatedEntries, changes, updatedChecksumEntries, options);
    log.info(t0, `Created empty file index journal for unchanged directory ${directory}`);
    return [index, journal2, limitExceeded];
  }
  if (!options.journal) {
    const newIndex2 = await writeIndex(directory, filename, updatedEntries, options);
    log.info(t0, `Updated file index for directory ${directory}`);
    return [newIndex2, false, limitExceeded];
  }
  const indexExists = await access(filename).then(() => true, () => false);
  let newIndex = index;
  if (!indexExists) {
    newIndex = await writeIndex(directory, filename, [], options);
  }
  const journalChanges = options.checksum && interrupted ? onlyChangesWithSha1sum(changes) : changes;
  const journal = await createJournal(filename, newIndex, updatedEntries, journalChanges, updatedChecksumEntries, options);
  log.info(t0, `Created file index journal for directory ${directory}`);
  return [newIndex, journal, limitExceeded];
}
import { getIndexName } from "./utils.js";
import { getJournalFilename, applyJournal, removeJournal as removeJournal2 } from "./journal.js";
import { readIndexHead, readStream, readStreams } from "./read-stream.js";
const readJournal = callbackify(readJournalAsync);
import { matcherFns } from "./matcher.js";
import { statIndex } from "./stat.js";
import { prettyPrint } from "./pretty-print.js";
export {
  applyJournal,
  getIndexName,
  getJournalFilename,
  matcherFns,
  prettyPrint,
  readIndexHead,
  readJournal,
  readStream,
  readStreams,
  removeJournal2 as removeJournal,
  statIndex,
  update
};
//# sourceMappingURL=index.js.map
