import { Transform } from "stream";
import { through } from "@home-gallery/stream";
import Logger from "@home-gallery/logger";
const log = Logger("plugin.extractorStreamFactory");
class ExtractorStreamFactory {
  manager;
  storage;
  extensions;
  constructor(manager, storage, extensions) {
    this.manager = manager;
    this.storage = storage;
    this.extensions = extensions;
  }
  #functionToStream(task, extractor) {
    return through(function transform(entry, _, cb) {
      Promise.resolve(task(entry)).then(() => cb(null, entry)).catch((err) => {
        cb(new Error(`Extractor function task ${extractor.name} for phase ${extractor.phase} failed `, { cause: err }));
      });
    });
  }
  #testAndTaskToStream(task, plugin) {
    const test = task.test ? task.test : () => true;
    const end = task.end ? task.end : () => Promise.resolve();
    return through(function transform(entry, _, cb) {
      if (!test(entry)) {
        return cb(null, entry);
      }
      task.task(entry).then(() => cb(null, entry)).catch((err) => {
        cb(new Error(`Extractor task ${plugin.name} for phase ${plugin.phase} failed`, { cause: err }));
      });
    }, function flush(cb) {
      end().then(cb, cb);
    });
  }
  async toStream(plugin, extractor) {
    return extractor.create(this.storage).then((task) => {
      if (task instanceof Transform) {
        const logError = (err) => log.warn(err, `Extractor transform task ${extractor.name} for phase ${extractor.phase} failed`);
        return { stream: task.on("error", logError), extractor, plugin };
      } else if (typeof task == "function") {
        return { stream: this.#functionToStream(task, extractor), extractor, plugin };
      } else if (typeof task.task == "function") {
        return { stream: this.#testAndTaskToStream(task, extractor), extractor, plugin };
      } else {
        throw new Error(`Invalid extractor task from ${plugin.name} extractor ${extractor.name}`);
      }
    });
  }
  async getExtractorStreams() {
    const disabledExtensions = this.manager.getConfig().pluginManager?.disabled || [];
    const t0 = Date.now();
    const streams = [];
    for (let extension of this.extensions) {
      if (extension.type != "extractor") {
        continue;
      }
      const extractor = extension.extension;
      const name = `${extension.plugin.name}.${extension.type}.${extractor.name}`;
      if (disabledExtensions.includes(extension.plugin.name) || disabledExtensions.includes(name)) {
        log.debug(`Extractor ${extractor.name} from plugin ${extension.plugin.name} is disabled`);
        continue;
      }
      await this.toStream(extension.plugin, extractor).then((stream) => streams.push(stream)).catch((err) => {
        log.warn(err, `Failed to create extractor task of ${extractor.name} for phase ${extractor.phase || "file"}: ${err}. Skip it`);
      });
    }
    log.debug(t0, `Loaded ${streams.length} extractor tasks from plugins`);
    const tearDown = async () => {
      const tearDownExtractors = streams.filter((stream) => stream.extractor?.tearDown).map((stream) => stream.extractor).reverse();
      if (!tearDownExtractors.length) {
        return;
      }
      const t02 = Date.now();
      for (let plugin of tearDownExtractors) {
        await plugin.tearDown().catch((err) => log.warn(err, `Failed to tear down plugin ${plugin.name}: ${err}`));
      }
      log.debug(t02, `All ${tearDownExtractors.length} extractors where teared down: ${tearDownExtractors.map((p) => p.name).join(", ")}`);
    };
    return [streams, tearDown];
  }
}
export {
  ExtractorStreamFactory
};
//# sourceMappingURL=extractorStreamFactory.js.map
